#Copyright (c) 2013 SUSE LLC.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "TableCell",
          ]

#------------------------------------------------------------------------------

class TableCell(interface):
        """
        An interface used by cells in a table.
        """

        def get_columnSpan(self):
                return Atspi.TableCell.get_column_span(self.obj)
        _columnSpanDoc = \
                """
                Get the number of columns occupied by this cell.
                """
        columnSpan = property(fget=get_columnSpan, doc=_columnSpanDoc)

        def get_columnHeaderCells(self):
                return Atspi.TableCell.get_column_header_cells(self.obj)
        _columnHeaderCellsDoc = \
                """
                Get the column headers as an array of cell accessibles.
                """
        columnHeaderCells = property(fget=get_columnHeaderCells, doc=_columnHeaderCellsDoc)

        def get_rowSpan(self):
                return Atspi.TableCell.get_row_span(self.obj)
        _rowSpanDoc = \
                """
                Get the number of rows occupied by this cell.
                """
        rowSpan = property(fget=get_rowSpan, doc=_rowSpanDoc)

        def get_rowHeaderCells(self):
                return Atspi.TableCell.get_row_header_cells(self.obj)
        _rowHeaderCellsDoc = \
                """
                Get the row headers as an array of cell accessibles.
                """
        rowHeaderCells = property(fget=get_rowHeaderCells, doc=_rowHeaderCellsDoc)

        def get_position(self):
                return Atspi.TableCell.get_position(self.obj)
        _positionDoc = \
                """
                Returns the tabular position of this accessible.
                """
        position = property(fget=get_position, doc=_positionDoc)

        def getRowColumnSpan(self):
                """
                determine the row and column indices and span of the given cell.
                """
                return Atspi.TableCell.get_row_column_span(self.obj)

        def get_table(self):
                return Atspi.TableCell.get_table(self.obj)
        _tableDoc = \
                """
                Returns a reference to the accessible of the containing table.
                """
        table = property(fget=get_table, doc=_tableDoc)

#END----------------------------------------------------------------------------



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "Table",
          ]

#------------------------------------------------------------------------------

class Table(interface):
        """
        An interface used by containers whose contained data is arranged
        in a "tabular" (i.e. row-column) fashion. Tables may resemble
        a two-dimensional grid, as in a spreadsheet, or may feature objects
        which span multiple rows and/or columns, but whose bounds are
        aligned on a row/column matrix. Thus, the Table interface may
        be used to represent "spreadsheets" as well as "frames".
        Objects within tables are children of the Table instance, and
        they may be referenced either via a child index or via a row/column
        pair. Their role may be ROLE_TABLE_CELL, but table 'cells' may
        have other roles as well. These 'cells' may implement other interfaces,
        such as Text, Action, Image, and Component, and should do so
        as appropriate to their onscreen representation and/or behavior.
        """

        def addColumnSelection(self, column):
                """
                Select the specified column, adding it to the current column
                selection, if the table's selection model permits it.
                @param : column
                @return True if the specified column was successfully selected,
                False if not.
                """
                return Atspi.Table.add_column_selection(self.obj, column)

        def addRowSelection(self, row):
                """
                Select the specified row, adding it to the current row selection,
                if the table's selection model permits it.
                @param : row
                @return True if the specified row was successfully selected,
                False if not.
                """
                return Atspi.Table.add_row_selection(self.obj, row)

        def getAccessibleAt(self, row, column):
                """
                Get the table cell at the specified row and column indices. 
                @param : row
                the specified table row, zero-indexed. 
                @param : column
                the specified table column, zero-indexed.
                @return an Accessible object representing the specified table
                cell.
                """
                return Atspi.Table.get_accessible_at(self.obj, row, column)

        def getColumnAtIndex(self, index):
                """
                Get the table column index occupied by the child at a particular
                1-D child index.
                @param : index
                the specified child index, zero-indexed.
                @return a long integer indicating the first column spanned by
                the child of a table, at the specified 1-D (zero-offset) index.
                """
                return Atspi.Table.get_column_at_index(self.obj, index);

        def getColumnDescription(self, column):
                """
                Get a text description of a particular table column. This differs
                from AccessibleTable_getColumnHeader, which returns an Accessible.
                @param : column
                the specified table column, zero-indexed.
                @return a UTF-8 string describing the specified table column,
                if available.
                """
                return Atspi.Table.get_column_description(self.obj, column)

        def getColumnExtentAt(self, row, column):
                """
                Get the number of columns spanned by the table cell at the specific
                row and column. (some tables can have cells which span multiple
                rows and/or columns).
                The returned values are meaningful only if the Table has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : row
                the specified table row, zero-indexed. 
                @param : column
                the specified table column, zero-indexed.
                @return a long integer indicating the number of columns spanned
                by the specified cell.
                """
                return Atspi.Table.get_column_extent_at(self.obj, row, column)

        def getColumnHeader(self, index):
                """
                Get the header associated with a table column, if available,
                as an instance of Accessible. This differs from getColumnDescription,
                which returns a string.
                @param : column
                the specified table column, zero-indexed.
                @return an Accessible representatin of the specified table column,
                if available.
                """
                return Atspi.Table.get_column_header(self.obj, index)

        def getIndexAt(self, row, column):
                """
                Get the 1-D child index corresponding to the specified 2-D row
                and column indices. 
                @param : row
                the specified table row, zero-indexed. 
                @param : column
                the specified table column, zero-indexed.
                @return a long integer which serves as the index of a specified
                cell in the table, in a form usable by Accessible::getChildAtIndex.
                """
                return Atspi.Table.get_index_at(self.obj, row, column)

        def getRowAtIndex(self, index):
                """
                Get the table row index occupied by the child at a particular
                1-D child index.
                @param : index
                the specified child index, zero-indexed.
                @return a long integer indicating the first row spanned by the
                child of a table, at the specified 1-D (zero-offset) index.
                """
                return Atspi.Table.get_row_at_index(self.obj, index)

        def getRowColumnExtentsAtIndex(self, index):
                """
                Given a child index, determine the row and column indices and
                extents, and whether the cell is currently selected. If the child
                at index is not a cell (for instance, if it is a summary, caption,
                etc.), False is returned.
                The returned values are meaningful only if the Table has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : index
                the index of the Table child whose row/column extents are requested.
                @param : row
                back-filled with the first table row associated with the cell
                with child index index. 
                @param : col
                back-filled with the first table column associated with the cell
                with child index index. 
                @param : row_extents
                back-filled with the number of table rows across which child
                i extends. 
                @param : col_extents
                back-filled with the number of table columns across which child
                i extends. 
                @param : is_selected
                a boolean which is back-filled with True if the child at index
                i corresponds to a selected table cell, False otherwise.
                Example: If the Table child at index '6' extends across columns
                5 and 6 of row 2 of a Table instance, and is currently selected,
                then retval=table::getRowColumnExtentsAtIndex(6,row,col,
                row_extents,
                col_extents,
                is_selected);
                 will return True, and after the call row, col, row_extents,
                col_extents, and is_selected will contain 2, 5, 1, 2, and True,
                respectively.
                @return True if the index is associated with a valid table cell,
                False if the index does not correspond to a cell. If False is
                returned, the values of the out parameters are undefined.
                """
                return Atspi.Table.get_row_column_extents_at_index(self.obj, index)

        def getRowDescription(self, index):
                """
                Get a text description of a particular table row. This differs
                from AccessibleTable_getRowHeader, which returns an Accessible.
                @param : row
                the specified table row, zero-indexed.
                @return a UTF-8 string describing the specified table row, if
                available.
                """
                return Atspi.Table.get_row_description(self.obj, index)

        def getRowExtentAt(self, row, column):
                """
                Get the number of rows spanned by the table cell at the specific
                row and column. (some tables can have cells which span multiple
                rows and/or columns).
                The returned values are meaningful only if the Table has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : row
                the specified table row, zero-indexed. 
                @param : column
                the specified table column, zero-indexed.
                @return a long integer indicating the number of rows spanned
                by the specified cell.
                """
                return Atspi.Table.get_row_extent_at(self.obj, row, column)

        def getRowHeader(self, row):
                """
                Get the header associated with a table row, if available. This
                differs from getRowDescription, which returns a string.
                @param : row
                the specified table row, zero-indexed.
                @return an Accessible representatin of the specified table row,
                if available.
                """
                return Atspi.Table.get_row_header(self.obj, row)

        def getSelectedColumns(self):
                """
                Obtain the indices of all columns which are currently selected.
                @return a sequence of integers comprising the indices of columns
                currently selected.
                """
                return Atspi.Table.get_selected_columns(self.obj)

        def getSelectedRows(self):
                """
                Obtain the indices of all rows which are currently selected.
                @return a sequence of integers comprising the indices of rows
                currently selected.
                """
                return Atspi.Table.get_selected_rows(self.obj)

        def isColumnSelected(self, column):
                """
                Determine whether a table column is selected. 
                @param : column
                the column being queried.
                @return True if the specified column is currently selected, False
                if not.
                """
                return Atspi.Table.is_column_selected(self.obj, column)

        def isRowSelected(self, row):
                """
                Determine whether a table row is selected. 
                @param : row
                the row being queried.
                @return True if the specified row is currently selected, False
                if not.
                """
                return Atspi.Table.is_row_selected(self.obj, row)

        def isSelected(self, row, column):
                """
                Determine whether the cell at a specific row and column is selected.
                @param : row
                a row occupied by the cell whose state is being queried. 
                @param : column
                a column occupied by the cell whose state is being queried.
                @return True if the specified cell is currently selected, False
                if not.
                """
                return Atspi.Table.is_selected(self.obj, row, column)

        def removeColumnSelection(self, column):
                """
                Remove the specified column from current column selection, if
                the table's selection model permits it.
                @param : column
                @return True if the specified column was successfully de-selected,
                False if not.
                """
                return Atspi.Table.remove_column_selection(self.obj, column)

        def removeRowSelection(self, row):
                """
                Remove the specified row from current row selection, if the table's
                selection model permits it.
                @param : row
                @return True if the specified row was successfully de-selected,
                False if not.
                """
                return Atspi.Table.remove_row_selection(self.obj, row)

        def get_caption(self):
                return Atspi.Table.get_caption(self.obj)
        _captionDoc = \
                """
                An Accessible which represents of a caption for a Table.
                """
        caption = property(fget=get_caption, doc=_captionDoc)

        def get_nColumns(self):
                return Atspi.Table.get_n_columns(self.obj)
        _nColumnsDoc = \
                """
                The total number of columns in this table (including empty columns),
                exclusive of columns which are programmatically hidden. Columns
                which are scrolled out of view or clipped by the current viewport
                are included.
                """
        nColumns = property(fget=get_nColumns, doc=_nColumnsDoc)

        def get_nRows(self):
                return Atspi.Table.get_n_rows(self.obj)
        _nRowsDoc = \
                """
                The total number of rows in this table (including empty rows),
                exclusive of any rows which are programmatically hidden. Rows
                which are merely scrolled out of view are included.
                """
        nRows = property(fget=get_nRows, doc=_nRowsDoc)

        def get_nSelectedColumns(self):
                return Atspi.Table.get_n_selected_columns(self.obj)
        _nSelectedColumnsDoc = \
                """
                The number of columns currently selected. A selected column is
                one in which all included cells are selected.
                """
        nSelectedColumns = property(fget=get_nSelectedColumns, doc=_nSelectedColumnsDoc)

        def get_nSelectedRows(self):
                return Atspi.Table.get_n_selected_rows(self.obj)
        _nSelectedRowsDoc = \
                """
                The number of rows currently selected. A selected row is one
                in which all included cells are selected.
                """
        nSelectedRows = property(fget=get_nSelectedRows, doc=_nSelectedRowsDoc)

        def get_summary(self):
                return Atspi.Table.get_summary(self.obj)
        _summaryDoc = \
                """
                An accessible object which summarizes the contents of a Table.
                This object is frequently itself a Table instance, albeit a simplified
                one.
                """
        summary = property(fget=get_summary, doc=_summaryDoc)

#END----------------------------------------------------------------------------



#!/usr/bin/python

# magFocusTracker
#
# Copyright 2009 Sun Microsystems Inc.
# Copyright 2010 Willie Walker
# Copyright 2011-2012 Igalia, S. L.
# Copyright 2011-2012 Inclusive Design Research Centre, OCAD University
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.
#
# * Contributor: Joanie Diggs <diggs@igalia.com>
# * Contributor: Joseph Scheuhammer <clown@alum.mit.edu>

"""Proof-of-concept standalone application that shows how:
1. to track keyboard focus and the caret using AT-SPI events, and
2. use D-Bus to drive the magnifier to insure the tracked object is
   within the magnified view.
"""

__copyright__ = \
  "Copyright (c) 2009 Sun Microsystems Inc." \
  "Copyright (c) 2010 Willie Walker" \
  "Copyright (c) 2011-2012 Igalia, S.L." \
  "Copyright (c) 2011-2012 Inclusive Design Research Centre, OCAD University"
__license__   = "LGPL"

import dbus
import pyatspi
import sys
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import Gdk
from gi.repository.Gio import Settings

_screenWidth = 0
_screenHeight = 0
_magnifier = None
_zoomer = None

class RoiHandler:
    """For handling D-Bus calls to zoomRegion.getRoi() asynchronously"""

    def __init__(self, left=0, top=0, width=0, height=0, centerX=0, centerY=0,
                 edgeMarginX=0, edgeMarginY=0):
        self.left = left
        self.top = top
        self.width = width
        self.height = height
        self.centerX = centerX
        self.centerY = centerY
        self.edgeMarginX = edgeMarginX
        self.edgeMarginY = edgeMarginY

    def setRoiCenter(self, reply):
        """Given a region of interest, put that at the center of the magnifier.

        Arguments:
        - reply:  an array defining a rectangle [left, top, right, bottom]
        """
        roiWidth = reply[2] - reply[0]
        roiHeight = reply[3] - reply[1]
        if self.width > roiWidth:
            self.centerX = self.left
        if self.height > roiHeight:
            self.centerY = self.top
        _setROICenter(self.centerX, self.centerY)

    def setRoiCursorPush(self, reply):
        """Given a region of interest, nudge it if the caret or control is not
        visible.

        Arguments:
        - reply:  an array defining a rectangle [left, top, right, bottom]
        """

        roiLeft = reply[0]
        roiTop = reply[1]
        roiWidth = reply[2] - roiLeft
        roiHeight = reply[3] - roiTop
        leftOfROI = (self.left - self.edgeMarginX) <= roiLeft
        rightOfROI = \
            (self.left + self.width + self.edgeMarginX) >= (roiLeft + roiWidth)
        aboveROI = (self.top - self.edgeMarginY)  <= roiTop
        belowROI = \
            (self.top + self.height + self.edgeMarginY) >= (roiTop + roiHeight)

        x1 = roiLeft
        x2 = roiLeft + roiWidth
        y1 = roiTop
        y2 = roiTop + roiHeight

        if leftOfROI:
            x1 = max(0, self.left - self.edgeMarginX)
            x2 = x1 + roiWidth
        elif rightOfROI:
            self.left = min(_screenWidth, self.left + self.edgeMarginX)
            if self.width > roiWidth:
                x1 = self.left
                x2 = x1 + roiWidth
            else:
                x2 = self.left + self.width
                x1 = x2 - roiWidth

        if aboveROI:
            y1 = max(0, self.top - self.edgeMarginY)
            y2 = y1 + roiHeight
        elif belowROI:
            self.top = min(_screenHeight, self.top + self.edgeMarginY)
            if self.height > roiHeight:
                y1 = self.top
                y2 = y1 + roiHeight
            else:
                y2 = self.top + self.height
                y1 = y2 - roiHeight

        _setROICenter((x1 + x2) / 2, (y1 + y2) / 2)

    def setRoiCenterErr(self, error):
        _dbusCallbackError('_setROICenter()', error)

    def setRoiCursorPushErr(self, error):
        _dbusCallbackError('_setROICursorPush()', error)

    def magnifyAccessibleErr(self, error):
        _dbusCallbackError('magnifyAccessible()', error)

def _dbusCallbackError(funcName, error):
    """Log D-Bus errors

    Arguments:
    - funcName: The name of the gsmag function that made the D-Bus call.
    - error: The error that D-Bus returned.
    """
    logLine = funcName + ' failed: ' + str(error)
    debug.println(debug.LEVEL_WARNING, logLine)

def _setROICenter(x, y):
    """Centers the region of interest around the given point.

    Arguments:
    - x: integer in unzoomed system coordinates representing x component
    - y: integer in unzoomed system coordinates representing y component
    """
    _zoomer.shiftContentsTo(x, y, ignore_reply=True)

def _setROICursorPush(x, y, width, height):
    """Nudges the ROI if the caret or control is not visible.

    Arguments:
    - x: integer in unzoomed system coordinates representing x component
    - y: integer in unzoomed system coordinates representing y component
    - width: integer in unzoomed system coordinates representing the width
    - height: integer in unzoomed system coordinates representing the height
    """

    roiPushHandler = RoiHandler(x, y, width, height)
    _zoomer.getRoi(reply_handler=roiPushHandler.setRoiCursorPush,
                   error_handler=roiPushHandler.setRoiCursorPushErr)

def magnifyAccessible(event, obj=None, extents=None):
    """Sets the region of interest to the upper left of the given
    accessible, if it implements the Component interface.  Otherwise,
    does nothing.

    Arguments:
    - event: the Event that caused this to be called
    - obj: the accessible
    """

    if event.type.startswith("object:state-changed") and not event.detail1:
        # This object just became unselected or unfocused, and we're not
        # big on nostalgia.
        return

    obj = obj or event.source

    haveSomethingToMagnify = False

    if extents:
        [x, y, width, height] = extents
        haveSomethingToMagnify = True
    elif event and event.type.startswith("object:text-caret-moved"):
        try:
            text = obj.queryText()
            if text and (text.caretOffset >= 0):
                offset = text.caretOffset
                if offset == text.characterCount:
                    offset -= 1
                [x, y, width, height] = \
                    text.getCharacterExtents(offset, 0)
                haveSomethingToMagnify = (width + height > 0)
        except:
            haveSomethingToMagnify = False

        if haveSomethingToMagnify:
            _setROICursorPush(x, y, width, height)
            return

    if not haveSomethingToMagnify:
        try:
            extents = obj.queryComponent().getExtents(0)
            [x, y, width, height] = \
                [extents.x, extents.y, extents.width, extents.height]
            haveSomethingToMagnify = True
        except:
            haveSomethingToMagnify = False

    if haveSomethingToMagnify:
        _setROICursorPush(x, y, width, height)

def startTracking():
    global _screenWidth
    global _screenHeight
    global _magnifier
    global _zoomer

    if _magnifier and _zoomer:
        screen = Gdk.Screen.get_default()
        _screenWidth = screen.width()
        _screenHeight = screen.height()

        pyatspi.Registry.registerEventListener(magnifyAccessible,
                                               "object:text-caret-moved",
                                               "object:state-changed:focused",
                                               "object:state-changed:selected")

def stopTracking():
    pyatspi.Registry.deregisterEventListener(magnifyAccessible,
                                             "object:text-caret-moved",
                                             "object:state-changed:focused",
                                             "object:state-changed:selected")

def onEnabledChanged(gsetting, key):
    if key != 'screen-magnifier-enabled':
        return

    enabled = gsetting.get_boolean(key)
    if enabled:
        startTracking()
    else:
        stopTracking()

def _initMagDbus():
    global _magnifier
    global _zoomer

    available = False
    try:
        bus = dbus.SessionBus(mainloop=DBusGMainLoop())
        proxy = \
          bus.get_object('org.gnome.Magnifier', '/org/gnome/Magnifier')
        _magnifier = dbus.Interface(proxy, 'org.gnome.Magnifier')
        zoomerPaths = _magnifier.getZoomRegions()
        if zoomerPaths:
            proxy = bus.get_object('org.gnome.Magnifier', zoomerPaths[0])
            _zoomer = dbus.Interface(proxy, 'org.gnome.Magnifier.ZoomRegion')
            available = True
    except:
        available = False

    return available

def main():
    magServiceAvailable = _initMagDbus()
    if magServiceAvailable:
        a11yAppSettings = Settings('org.gnome.desktop.a11y.applications')
        a11yAppSettings.connect('changed', onEnabledChanged)
        if a11yAppSettings.get_boolean('screen-magnifier-enabled'):
            startTracking()
        pyatspi.Registry.start()
    else:
        print('Magnification service not available. Exiting.')

    return 0

if __name__ == "__main__":
    sys.exit(main())



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi
from pyatspi import StateSet

st = [pyatspi.STATE_MULTI_LINE,
      pyatspi.STATE_MODAL,
      pyatspi.STATE_INDETERMINATE,
      pyatspi.STATE_SUPPORTS_AUTOCOMPLETION,
      pyatspi.STATE_VERTICAL,]

def _createNode(doc, accessible, parentElement):
	e = doc.createElement("accessible")

	nameA = doc.createAttribute('name')
	roleA = doc.createAttribute('role')
	descA = doc.createAttribute('description')
	e.setAttributeNode(nameA)
	e.setAttributeNode(roleA)
	e.setAttributeNode(descA)
	e.setAttribute("name", accessible.name)
	e.setAttribute("role", str(int(accessible.getRole())))
	e.setAttribute("description", accessible.description)

	for i in range(0, accessible.childCount):
		_createNode(doc, accessible.getChildAtIndex(i), e)

	parentElement.appendChild(e)

class AccessibleTest(_PasyTest):

	__tests__ = ["setup",
		     "test_name",
		     "test_getChildAtIndex",
		     "test_isEqual",
		     "test_getApplication",
		     "test_getAttributes",
		     "test_parent",
		     "test_getIndexInParent",
		     "test_getLocalizedRoleName",
		     "test_getRelationSet",
		     "test_getRole",
		     "test_getRoleName",
		     "test_getState",
		     "test_childCount",
		     "test_description",
		     "test_tree",
		     "test_null_interface",
		     "teardown",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "Accessible", False)
		self._bus = bus
		self._path = path

	def setup(self, test):
		self._registry = pyatspi.Registry()
		self._desktop = self._registry.getDesktop(0)
		self._root = pyatspi.findDescendant (self._desktop, lambda x: x.name == "atspi-test-main" and x.getRole() == pyatspi.ROLE_WINDOW)

	def test_name(self, test):
		root = self._root
		test.assertEqual(root.name, "atspi-test-main", "Expected name - \"main\". Recieved - \"%s\"" % (root.name,))

	def test_getChildAtIndex(self, test):
		root = self._root
		a = root.getChildAtIndex(0)
		test.assertEqual(a.name, "gnome-settings-daemon",
					 "Expected name - \"gnome-settings-daemon\". Recieved - \"%s\"" % (a.name,))
		b = root.getChildAtIndex(1)
		test.assertEqual(b.name, "gnome-panel",
					 "Expected name - \"gnome-panel\". Recieved - \"%s\"" % (b.name,))
		c = root.getChildAtIndex(2)
		test.assertEqual(c.name, "nautilus",
					 "Expected name - \"nautilus\". Recieved - \"%s\"" % (c.name,))
		
	def test_isEqual(self, test):
		root = self._root

		a = root.getChildAtIndex(1)
		if not a.isEqual(a):
			test.fail("Same accessible found unequal to self")

		b = root.getChildAtIndex(1)
		if not a.isEqual(b):
			test.fail("Similar accessibles found unequal")
		if not b.isEqual(a):
			test.fail("Similar accessibles found unequal")

		c = root.getChildAtIndex(2)
		if c.isEqual(a):
			test.fail("Different accessibles found equal")
		if a.isEqual(c):
			test.fail("Different accessibles found equal")

	def test_getApplication(self, test):
		root = self._root
		application = root.getApplication()
		if not root.isEqual(application):
			test.fail("Root accessible does not provide itself as its Application")

		a = root.getChildAtIndex(1)
		application = a.getApplication()
		if not root.isEqual(application):
			test.fail("Child accessible does not provide the root as its Application")


	def test_getAttributes(self, test):
		root = self._root
		attr = root.getAttributes()
		res = ["foo:bar", "baz:qux", "quux:corge"]
		attr.sort()
		res.sort()
		test.assertEqual(attr, res, "Attributes expected %s, recieved %s" % (res, attr))

	def test_parent(self, test):
		root = self._root

		a = root.getChildAtIndex(1)
		pa = a.parent
		if not root.isEqual(pa):
			test.fail("Child does not correctly report its parent")

	def test_getIndexInParent(self, test):
		root = self._root

		for i in range(0, root.childCount):
			child = root.getChildAtIndex(i)
			test.assertEqual(i, child.getIndexInParent(), "Childs index in parent reported incorrectly")

	def test_getLocalizedRoleName(self, test):
		root = self._root

		ans = "window"
		res = root.getLocalizedRoleName()
		test.assertEqual(ans, res,
				 "Expected LocalizedRoleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

		a = root.getChildAtIndex(1)
		a = a.getChildAtIndex(0)
		ans = "html container"
		res = a.getLocalizedRoleName()
		test.assertEqual(ans, res,
				 "Expected LocalizedRoleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

	def test_getRelationSet(self, test):
		root = self._root
		# Complete test of Relation interface is separate
		rset = root.getRelationSet()

	def test_getRole(self, test):
		root = self._root
		test.assertEqual(root.getRole(), 69,
				 "Expected role - \"69\". Recieved - \"%d\"" % (int(root.getRole()),))

	def test_getRoleName(self, test):
		root = self._root

		ans = "window"
		res = root.getRoleName()
		test.assertEqual(ans, res,
				 "Expected roleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

		a = root.getChildAtIndex(1)
		a = a.getChildAtIndex(0)
		ans = "html container"
		res = a.getRoleName()
		test.assertEqual(ans, res,
				 "Expected roleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

	def test_getState(self, test):
		root = self._root
		state = root.getState()
		res = StateSet(*st)
		if not res.equals(state):
			test.fail("States not reported correctly")

	def test_childCount(self, test):
		root = self._root
		test.assertEqual(root.childCount, 11,
				 "Expected role - \"11\". Recieved - \"%d\"" % (root.childCount,))

	def test_description(self, test):
		root = self._root
		description = "The main accessible object, root of the accessible tree"
		test.assertEqual(root.description, description,
				 "Expected description - \"%s\". Recieved - \"%s\"" % (description, root.description,))

	def test_tree(self, test):
		"""
		This is a mild stress test for the 
		methods:

		getChildAtIndex
		
		And the attributes:

		name
		description

		It checks a tree of these values is correctly
		passed from Application to AT.
		"""
		root = self._root

		doc = minidom.Document()
		_createNode(doc, root, doc)
		answer = doc.toprettyxml()


		correct = os.path.join(os.environ["TEST_DATA_DIRECTORY"],
					"accessible-test-results.xml")
		file = open(correct)
		cstring = file.read()

		correct2 = os.path.join(os.environ["TEST_DATA_DIRECTORY"],
					"accessible-test-results-stable.xml")
		file = open(correct2)
		cstring2 = file.read()
		
		if answer != cstring and \
		   answer != cstring2:
		    test.fail("Object tree not passed correctly")

	def test_null_interface(self, test):
		root = self._root
		try:
			text = root.queryText()
		except NotImplementedError:
			return
		test.fail ("Should throw NotImplementedError")

	def teardown(self, test):
		pass



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi

class RelationTest(_PasyTest):

	__tests__ = ["setup",
		     "teardown",
		     "test_getRelationType",
		     "test_getRelationTypeName",
		     "test_getNTargets",
		     "test_getTarget",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "Relation", False)
		self._bus = bus
		self._path = path

	def setup(self, test):
		self._registry = pyatspi.Registry()
		self._desktop = self._registry.getDesktop(0)
		self._root = pyatspi.findDescendant (self._desktop, lambda x: x.name == "atspi-test-main" and x.getRole() == pyatspi.ROLE_APPLICATION)
		self._rset = self._root.getRelationSet()
		test.assertEqual(len(self._rset), 4, "Num relations expected %d, recieved %d" % (4, len(self._rset)))

	def test_getRelationType(self, test):
		expected = [pyatspi.RELATION_EMBEDS,
			    pyatspi.RELATION_PARENT_WINDOW_OF,
			    pyatspi.RELATION_DESCRIBED_BY,
			    pyatspi.RELATION_POPUP_FOR,]
		rtypes = [rel.getRelationType() for rel in self._rset]
		for exp, type in zip(expected, rtypes):
			test.assertEqual(exp, type, "Relation type expected %s, recieved %s" % (exp, type))

	def test_getRelationTypeName(self, test):
		# FIXME This may not have been implemented in CORBA.
		#       Completely unused?
		pass

	def test_getNTargets(self, test):
		expected = [1, 1, 1, 3]
		ntargs = [rel.getNTargets() for rel in self._rset]
		for exp, ntarg in zip(expected, ntargs):
			test.assertEqual(exp, ntarg, "Number of targets expected %s, recieved %s" % (exp, ntarg))

	def test_getTarget(self, test):
		rone = self._rset[0]
		tone = rone.getTarget(0)
		tonename = tone.name
		test.assertEqual(tonename, "r1", "Target name expected %s, recieved %s" % ("r1", tonename))
		tonerole = tone.getRoleName()
		test.assertEqual(tonerole, "invalid", "Target RoleName expected %s, recieved %s" % ("invalid", tonename))

		rtwo = self._rset[3]
		ttwo = rtwo.getTarget(2)
		ttwoname = ttwo.name
		test.assertEqual(ttwoname, "m3", "Target name expected %s, recieved %s" % ("m3", ttwoname))
		ttworole = ttwo.getRoleName()
		test.assertEqual(ttworole, "invalid", "Target RoleName expected %s, recieved %s" % ("invalid", ttwoname))

	def teardown(self, test):
		pass



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "Hypertext",
          ]

#------------------------------------------------------------------------------

class Hypertext(interface):
        """
        An interface used for objects which implement linking between
        multiple resource or content locations, or multiple 'markers'
        within a single document. A Hypertext instance is associated
        with one or more Hyperlinks, which are associated with particular
        offsets within the Hypertext's included content.
        """

        def getLink(self, index):
                """
                Get one of the Hyperlinks associated with this Hypertext object,
                by index.
                @param : linkIndex
                an integer from 0 to getNLinks() - 1. 
                @return the Hyperlink in this Hypertext object.
                """
                return Atspi.Hypertext.get_link(self.obj, index)

        def getLinkIndex(self, character_index):
                """
                Get the hyperlink index, if any, associated with a particular
                character offset in the Hypertext object. For Hypertext implementors
                without textual content, all hyperlinks are associated with character
                offset '0'.
                @return the index of the Hyperlink associated with character
                offset characterIndex, or -1 if no Hyperlink is associated with
                that character offset.
                """
                return Atspi.Hypertext.get_link_index(self.obj, character_index)

        def getNLinks(self):
                """
                Query the hypertext object for the number of Hyperlinks it contains.
                @return the number of Hyperlinks associated with this Hypertext
                object, as a long integer.
                """
                return Atspi.Hypertext.get_n_links(self.obj)

#END----------------------------------------------------------------------------



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi
from pyatspi.Accessibility import Component
from pyatspi.utils import BoundingBox

ATSPI_LAYER_WIDGET = 3
ATSPI_LAYER_MDI = 4
ATSPI_LAYER_WINDOW = 7

extents_expected = [(0,0,30,20), (40,30,30,40), (0,0,70,70)]
sizes_expected = [(30,20), (30,40), (70,70)]
positions_expected = [(0,0), (40,30), (0,0)]
layers_expected = [ATSPI_LAYER_WINDOW, ATSPI_LAYER_WIDGET, ATSPI_LAYER_MDI]
zorders_expected = [-100, 100]

class ComponentTest(_PasyTest):

	__tests__ = ["setup",
		     "test_contains",
		     "test_getAccessibleAtPoint",
		     "test_getExtents",
		     "test_getPosition",
		     "test_getSize",
		     "test_getLayer",
		     "test_getMDIZOrder",
		     "test_grabFocus",
		     "test_registerFocusHandler",
		     "test_deregisterFocusHandler",
		     "test_getAlpha",
		     "teardown",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "Component", False)
		self._bus = bus
		self._path = path

	def setup(self, test):
		self._registry = pyatspi.Registry()
		self._desktop = self._registry.getDesktop(0)
		self._root = pyatspi.findDescendant (self._desktop, lambda x: x.name == "atspi-test-main" and x.getRole() == pyatspi.ROLE_APPLICATION)

	def test_contains(self, test):
		pass

	def test_getAccessibleAtPoint(self, test):
		pass

	def test_getExtents(self, test):
		root = self._root
		one = root.getChildAtIndex(0)
		two = root.getChildAtIndex(1)

		comps = [one.queryComponent(),
			 two.queryComponent(),
			 root.queryComponent(),]
 
		for expected, comp in zip(extents_expected, comps):
			extents = comp.getExtents(0)
			test.assertEqual(extents, BoundingBox(*expected), 
					 "Extents not correct. Expected (%d, %d, %d, %d), Recieved (%d, %d, %d, %d)"
					 % (expected[0], expected[1], expected[2], expected[3], 
						extents[0], extents[1], extents[2], extents[3]))

	def test_getPosition(self, test):
		root = self._root
		one = root.getChildAtIndex(0)
		two = root.getChildAtIndex(1)

		comps = [one.queryComponent(),
			 two.queryComponent(),
			 root.queryComponent(),]
 
		for expected, comp in zip(positions_expected, comps):
			position = comp.getPosition(0)
			test.assertEqual(position, expected, 
					 "Position not correct. Expected (%d, %d) Recieved (%d, %d)"
					 % (expected[0], expected[1], position[0], position[1]))

	def test_getSize(self, test):
		root = self._root
		one = root.getChildAtIndex(0)
		two = root.getChildAtIndex(1)

		comps = [one.queryComponent(),
			 two.queryComponent(),
			 root.queryComponent(),]
 
		for expected, comp in zip(sizes_expected, comps):
			size = comp.getSize()
			test.assertEqual(size, expected, 
					 "Size not correct. Expected (%d, %d) Recieved (%d, %d)"
					 % (expected[0], expected[1], size[0], size[1]))

	def test_getLayer(self, test):
		root = self._root
		one = root.getChildAtIndex(0)
		two = root.getChildAtIndex(1)

		comps = [one.queryComponent(),
			 two.queryComponent(),
			 root.queryComponent(),]
 
		for expected, comp in zip(layers_expected, comps):
			layer = comp.getLayer()
			test.assertEqual(layer, expected, 
					 "Layer not correct. Expected %d, Recieved %d"
					 % (int(layer), int(expected)))

	def test_getMDIZOrder(self, test):
		root = self._root
		one = root.getChildAtIndex(0)
		two = root.getChildAtIndex(1)

		comps = [two.queryComponent(),
			 root.queryComponent(),]
 
		for expected, comp in zip(zorders_expected, comps):
			mdizo = comp.getMDIZOrder()
			test.assertEqual(mdizo, expected, 
					 "ZOrder not correct. Expected %d, Recieved %d"
					 % (expected, mdizo))

	def test_grabFocus(self, test):
		pass

	def test_registerFocusHandler(self, test):
		pass

	def test_deregisterFocusHandler(self, test):
		pass

	def test_getAlpha(self, test):
		pass

	def teardown(self, test):
		pass



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi

class ActionTest(_PasyTest):

	__tests__ = ["setup",
		     "test_nActions",
		     "test_getDescription",
		     "test_getName",
		     "test_doAction",
		     "test_getKeyBinding",
		     "teardown",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "Action", False)
		self._bus = bus
		self._path = path

	def setup(self, test):
		self._registry = pyatspi.Registry()
		import time
		self._desktop = self._registry.getDesktop(0)
		print("--desktop len", len(self._desktop))
		for i in self._desktop:
			try:
				print("-- object",i,i.getRole())
			except:
				pass
		self._root = pyatspi.findDescendant (self._desktop, lambda x: x.name == "atspi-test-main" and x.getRole() == pyatspi.ROLE_APPLICATION)
		print("--root", self._root)

	def test_nActions(self, test):
		root = self._root
		root = root.queryAction()
		nact = root.nActions
		test.assertEqual(nact, 10, "nActions expected %d, recieved %d" % (10, nact))

	def test_getName(self, test):
		root = self._root
		root = root.queryAction()
		name = root.getName(0)
		test.assertEqual(name, "First action", "Name expected %s, recieved %s" % ("First action", name))
		name = root.getName(1)
		test.assertEqual(name, "Action", "Name expected %s, recieved %s" % ("Action", name))

	def test_getDescription(self, test):
		root = self._root
		root = root.queryAction()
		description = root.getDescription(0)
		expected = "First action performed"
		test.assertEqual(description, expected, "Description expected %s, recieved %s" % (expected, description))
		description = root.getDescription(1)
		expected = "Description of action"
		test.assertEqual(description, expected, "Description expected %s, recieved %s" % (expected, description))

	def test_doAction(self, test):
		root = self._root
		root = root.queryAction()
		#TODO have event emitted to check action has been performed
		for i in range(0, root.nActions):
			root.doAction(i)

	def test_getKeyBinding(self, test):
		root = self._root
		root = root.queryAction()
		for i in range(0, root.nActions):
			keybinding = root.getKeyBinding(i)
			expected = "%s" % (i,)
			test.assertEqual(keybinding, expected,
					 "Keybinding expected %s, recieved %s" % (expected, keybinding))

	def teardown(self, test):
		pass



#!/usr/bin/python
#
# keypress.py
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.
#
# An pyatspi2 example to demonstate a client listener for
# Keypresses and releases in pyatspi2

import pyatspi

# Callback to print the active window on key press amd filter out the key release
def on_key_input(event):

	if event.type == pyatspi.KEY_RELEASED_EVENT:
		return False
	if event.event_string=='F4':
		pyatspi.Registry.stop()
		return True
	if event.event_string =='F3':
		print_tree(0, active_window())
		return True
	help_message()

def active_window():
	desktop = pyatspi.Registry.getDesktop(0)
	for app in desktop:
		for window in app:
			if window.getState().contains(pyatspi.STATE_ACTIVE):
				return window

# Print hierarchy tree.
def print_tree(level,root):
	print ('%s-> %s' % (' ' * level, root))
	for tree in root:
		print_tree(level+1, tree)

def help_message():
	print('Press F3 to print the accessible hierarchy for the active window.\nPress F4 to exit.')

help_message()
pyatspi.Registry.registerKeystrokeListener(on_key_input, kind=(pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT))
pyatspi.Registry.start()



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi
from pyatspi import StateSet

os = [pyatspi.STATE_MULTISELECTABLE,
      pyatspi.STATE_PRESSED,
      pyatspi.STATE_SHOWING,
      pyatspi.STATE_TRANSIENT,
      pyatspi.STATE_COLLAPSED,
      pyatspi.STATE_EDITABLE,]

class StateTest(_PasyTest):

	__tests__ = ["setup",
		     "test_contains",
		     "test_add",
		     "test_remove",
		     "test_equals",
		     "test_compare",
		     "test_isEmpty",
		     "test_getStates",
		     "teardown",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "State", False)

	def setup(self, test):
		pass

	def test_contains(self, test):
		state = StateSet(*os)
		if not state.contains(pyatspi.STATE_PRESSED):
			test.fail("Does not find contained state")
		if state.contains(pyatspi.STATE_ACTIVE):
			test.fail("Finds state not contained")

	def test_add(self, test):
		state = StateSet()
		state.add(pyatspi.STATE_PRESSED)
		if not state.contains(pyatspi.STATE_PRESSED):
			test.fail("State not added")

	def test_remove(self, test):
		state = StateSet(*os)
		state.remove(pyatspi.STATE_PRESSED)
		if state.contains(pyatspi.STATE_PRESSED):
			test.fail("State not removed")

	def test_equals(self, test):
		one = StateSet(*os)
		two = StateSet(*os)
		if not one.equals(two):
			test.fail("Same states not found equal")
		two.remove(pyatspi.STATE_PRESSED)
		if two.equals(one):
			test.fail("Unequal states found equal")

	def test_isEmpty(self, test):
		emp = StateSet()
		if not emp.isEmpty():
			test.fail("Empty state found non-empty")
		emp.add(pyatspi.STATE_PRESSED)
		if emp.isEmpty():
			test.fail("State incorrectly found empty")

	def test_compare(self, test):
		one = StateSet(*os)
		two = StateSet(*os)

		onemtwo = one.compare(two)
		if not onemtwo.isEmpty():
			test.fail("Equal states when compared yeilds non-empty state")

		one.add(pyatspi.STATE_ACTIVE)
		onemtwo = one.compare(two)
		
		act = StateSet(pyatspi.STATE_ACTIVE)
		if not onemtwo.equals(act):
			test.fail("Compared states do not yeild correct state")

	def test_getStates(self, test):
		state = StateSet(*os)

		states = state.getStates()
		cone = set(states)
		ctwo = set(os)

		if not (cone.issubset(ctwo) and ctwo.issubset(cone)):
			test.fail("States not reported correctly")

	def teardown(self, test):
		pass



#Copyright (C) 2008 Codethink Ltd

#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import GObject, GLib
from .Events import Events

import traceback

PASY_TEST_NOT_STARTED = 0
PASY_TEST_IN_PROGRESS = 1
PASY_TEST_FAIL = 2
PASY_TEST_WIN = 3

class PasyEvents(Events):
	__events__ = ('finished', )

class PasyTestStep(object):

	def __init__(self, name):
		self.events = PasyEvents()
		self._state = PASY_TEST_NOT_STARTED
		self.failMsg = None

		self._name = name

	def win(self):
		if self._state == PASY_TEST_IN_PROGRESS:
			self._state = PASY_TEST_WIN
			self.events.finished()
	
	def fail(self, msg):
		if self._state == PASY_TEST_IN_PROGRESS:
			self._state = PASY_TEST_FAIL
			self.failMsg = msg
			self.events.finished()

	def assertEqual(self, a, b, msg):
		if a != b:
			self.fail(msg)

	def assertNotEqual(self, a, b, msg):
		if a == b:
			self.fail(msg)

	def run(self):
		self._state = PASY_TEST_IN_PROGRESS
		self.entry()

	def report(self):
		if self._state == PASY_TEST_WIN:
			return "%s - PASSED" % (self._name,)
		elif self._state == PASY_TEST_FAIL:
			return "%s - FAILED - %s" % (self._name, self.failMsg)
		else:
			return "%s - INCOMPLETE" % (self._name,)

	@property
	def state(self):
		return self._state

class PasyTestFunc(PasyTestStep):
	
	def __init__(self, name, func):
		PasyTestStep.__init__(self, name)
		self._func = func

	def entry(self):
		try:
			self._func(self)
		except Exception as e:
			self.fail(str(e))
			traceback.print_exc()
		self.win()

class PasyTest(PasyTestStep):

	__tests__ = []

	def __init__(self, name, cont):
		PasyTestStep.__init__(self, name)

		self._cont = cont
		self._tests = []

		for name in self.__tests__:
			func = getattr(self, name)
			self._addfunc(name, func)

	def _addfunc(self, name, func):
		functest = PasyTestFunc(name, func)
		self._tests.append(functest)

	def entry(self):
		self._iter = self._test_iterator()
		GLib.idle_add(self.idle_handler)

	def idle_handler(self):
		try:
			step = next(self._iter)
			def finished_handler():
				if step.state == PASY_TEST_WIN or self._cont == True:
					GLib.idle_add(self.idle_handler)
				elif step.state == PASY_TEST_FAIL and self._cont == False:
					self.fail("Sub test %s Failed" % step._name)
			step.events.finished += finished_handler
			step.run()
		except StopIteration:
			# No More tests, check for success or fail
			succeeded = True
			for test in self._tests:
				succeeded = succeeded and (test.state == PASY_TEST_WIN)
			if succeeded:
				self.win()
			else:
				self.fail()
		return False

	def _test_iterator(self):
		for test in self._tests:
			yield test

	def report(self):
		if self._state == PASY_TEST_WIN:
			header =  "%s - PASSED" % (self._name,)
		elif self._state == PASY_TEST_FAIL:
			header =  "%s - FAILED" % (self._name,)
		else:
			header =  "%s - INCOMPLETE" %s (self._name,)

		step_messages = []
		for test in self._tests:
			step_messages.append(test.report())

		step_report =  "\n\t".join(step_messages)
		return header + "\n\t" + step_report



#Copyright (C) 2008 Codethink Ltd

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import string

class _ELessList(list):
        def __getitem__(self, index):
                try:
                        return list.__getitem__(self, index)
                except IndexError:
                        return None

class EventType(str):
        """
        Wraps the AT-SPI event type string so its components can be accessed 
        individually as klass (can't use the keyword class), major, minor, and detail 
        (klass_major_minor_detail).

        @note: All attributes of an instance of this class should be considered 
                public readable as it is acting a a struct.
        @ivar klass: Most general event type identifier (object, window, mouse, etc.)
        @type klass: string
        @ivar major: Second level event type description
        @type major: string
        @ivar minor: Third level event type description
        @type minor: string
        @ivar detail: Lowest level event type description
        @type detail: string
        @ivar name: Full, unparsed event name as received from AT-SPI
        @type name: string
        @cvar format: Names of the event string components
        @type format: 4-tuple of string
        """

        _SEPARATOR = ':'

        def __init__(self, name):
                """
                Parses the full AT-SPI event name into its components
                (klass:major:minor:detail). If the provided event name is an integer
                instead of a string, then the event is really a device event.

                @param name: Full AT-SPI event name
                @type name: string
                @raise AttributeError: When the given event name is not a valid string 
                """
                stripped = name.strip(self._SEPARATOR)
                separated = stripped.split(self._SEPARATOR, 3)
                self._separated = _ELessList(separated)

                self.klass = self._separated[0]
                self.major = self._separated[1]
                self.minor = self._separated[2]
                self.detail = self._separated[3]

        def is_subtype(self, event_type, excludeSelf = False):
                """
                Determines if the passed event type is a subtype
                of this event.
                """
                if event_type.klass and event_type.klass !=  self.klass:
                        return False
                else:
                        if event_type.major and event_type.major != self.major:
                                return False
                        else:
                                if event_type.minor and event_type.minor != self.minor:
                                        return False
                if (excludeSelf and event_type.klass == self.klass
                    and event_type.major == self.major and event_type.minor == self.minor):
                        return False
                return True

        @property
        def name(self):
                return str(self)

        @property
        def value(self):
                return str(self)



#
# Copyright 2008 Codethink Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi

class DesktopTest(_PasyTest):

	__tests__ = ["setup",
		     "test_name",
		     "test_getChildAtIndex",
		     "test_isEqual",
		     "test_getApplication",
		     "test_getAttributes",
		     "test_parent",
		     "test_getIndexInParent",
		     "test_getLocalizedRoleName",
		     "test_getRelationSet",
		     "test_getRole",
		     "test_getRoleName",
		     "test_getState",
		     "test_childCount",
		     "test_description",
		     "test_contains",
		     "test_getAccessibleAtPoint",
		     "test_getExtents",
		     "test_getPosition",
		     "test_getSize",
		     "test_getLayer",
		     "test_getMDIZOrder",
		     "test_grabFocus",
		     "test_registerFocusHandler",
		     "test_deregisterFocusHandler",
		     "test_getAlpha",
		     "teardown",
		     ]

	def __init__(self, bus, path):
		_PasyTest.__init__(self, "Desktop", False)
		self._bus = bus
		self._path = path

	def setup(self, test):
		self._registry = pyatspi.Registry()
		self._desktop = self._registry.getDesktop(0)

	def test_name(self, test):
		name = self._desktop.name
		test.assertEqual(name, "main", "Expected name - \"main\". Recieved - \"%s\"" % (name,))

	def test_getChildAtIndex(self, test):
		root = self._desktop.getChildAtIndex(0)
		test.assertEqual(root.name, "main",
					 "Expected name - \"main\". Recieved - \"%s\"" % (root.name,))
		
	def test_isEqual(self, test):
		if not self._desktop.isEqual(self._desktop):
			test.fail("Same accessible found unequal to self")

		root = self._desktop.getChildAtIndex(0)
		if root.isEqual(self._desktop):
			test.fail("Different accessibles found equal")
		if self._desktop.isEqual(root):
			test.fail("Different accessibles found equal")

	def test_getApplication(self, test):
		test.assertEqual(self._desktop.getApplication(), None,
				"Expected application - \"None\". Recieved - \"%s\"" % (self._desktop.getApplication(),))

	def test_getAttributes(self, test):
		test.assertEqual(self._desktop.getAttributes(), [],
				"Expected attributes - \"[]\". Recieved - \"%s\"" % (self._desktop.getAttributes(),))

	def test_parent(self, test):
		test.assertEqual(self._desktop.parent, None,
				"Expected parent - \"None\". Recieved - \"%s\"" % (self._desktop.parent,))

	def test_getIndexInParent(self, test):
		test.assertEqual(self._desktop.getIndexInParent(), -1,
				"Expected index - \"-1\". Recieved - \"%d\"" % (self._desktop.getIndexInParent(),))

	def test_getLocalizedRoleName(self, test):
		ans = "unknown"
		res = self._desktop.getLocalizedRoleName()
		test.assertEqual(ans, res,
				 "Expected LocalizedRoleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

	def test_getRelationSet(self, test):
		rset = self._desktop.getRelationSet()
		test.assertEqual(rset, [],
				 "Expected relation set - \"[]\". Recieved - \"%s\"" % (rset,))

	def test_getRole(self, test):
		test.assertEqual(self._desktop.getRole(), pyatspi.ROLE_UNKNOWN,
				 "Expected role - \"ROLE_UNKNOWN\". Recieved - \"%s\"" % (self._desktop.getRole(),))

	def test_getRoleName(self, test):
		ans = "unknown"
		res = self._desktop.getRoleName()
		test.assertEqual(ans, res,
				 "Expected RoleName - \"%s\". Recieved - \"%s\"" % (ans, res,))

	def test_getState(self, test):
		state = self._desktop.getState()
		res = pyatspi.StateSet()
		if not state.equals(res):
			test.fail("Desktop state set is not empty")

	def test_childCount(self, test):
		test.assertEqual(self._desktop.childCount, 1,
				 "Expected childCount - \"1\". Recieved - \"%d\"" % (self._desktop.childCount,))

	def test_description(self, test):
		description = ""
		test.assertEqual(self._desktop.description, description,
			"Expected description - \"%s\". Recieved - \"%s\"" % (description, self._desktop.description,))

	def test_contains(self, test):
		pass

	def test_getAccessibleAtPoint(self, test):
		pass

	def test_getExtents(self, test):
		comp = self._desktop.queryComponent()
 
		extents = comp.getExtents(0)
		expected = pyatspi.BoundingBox(*(0,0,1024, 768))
		test.assertEqual(extents, expected, 
				 "Extents not correct. Expected (%d, %d, %d, %d), Recieved (%d, %d, %d, %d)"
				 % (expected[0], expected[1], expected[2], expected[3], 
					extents[0], extents[1], extents[2], extents[3]))

	def test_getPosition(self, test):
		comp = self._desktop.queryComponent()

		position = comp.getPosition(0)
		test.assertEqual(position, (0,0), 
				 "Position not correct. Expected (%d, %d) Recieved (%d, %d)"
				 % (0, 0, position[0], position[1]))

	def test_getSize(self, test):
		comp = self._desktop.queryComponent()
 
		size = comp.getSize()
		test.assertEqual(size, (1024, 768), 
				 "Size not correct. Expected (%d, %d) Recieved (%d, %d)"
				 % (1024, 768, size[0], size[1]))

	def test_getLayer(self, test):
		comp = self._desktop.queryComponent()
 
		layer = comp.getLayer()
		test.assertEqual(layer, pyatspi.LAYER_WIDGET, 
				 "Layer not correct. Expected %d, Recieved %d"
				 % (layer, pyatspi.LAYER_WIDGET))

	def test_getMDIZOrder(self, test):
		comp = self._desktop.queryComponent()
 
		mdizo = comp.getMDIZOrder()
		test.assertEqual(mdizo, 0, 
				 "ZOrder not correct. Expected %d, Recieved %d"
				 % (0, mdizo))

	def test_grabFocus(self, test):
		pass

	def test_registerFocusHandler(self, test):
		pass

	def test_deregisterFocusHandler(self, test):
		pass

	def test_getAlpha(self, test):
		pass

	def teardown(self, test):
		pass



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "Action",
          ]

#------------------------------------------------------------------------------

class Action(interface):
        """
        An interface through which a user-actionable user interface component
        can be manipulated. Components which react to mouse or keyboard
        input from the user, (with the exception of pure text entry fields
        with no other function), should implement this interface. Typical
        actions include "click", "press", "release" (for instance for
        buttons), "menu" (for objects which have context menus invokable
        from mouse or keyboard), "open" for icons representing files
        folders, and others.
        """

        def getActions(self):
                """
                getActions:
                Retrieves all the actions at once.  
                @return : an array of an array of strings in the form
                [[name, description, keybinding], ...]
                """
                return Atspi.Action.get_actions(self.obj)

        def doAction(self, index):
                """
                doAction: 
                @param : index
                the 0-based index of the action to perform.
                Causes the object to perform the specified action.
                @return : a boolean indicating success or failure.
                """
                return Atspi.Action.do_action(self.obj, index)

        def getDescription(self, index):
                """
                getDescription: 
                @param : index
                the index of the action for which a description is desired.
                Get the description of the specified action. The description
                of an action may provide information about the result of action
                invocation, unlike the action name. 
                @return : a string containing the description of the specified
                action.
                """
                return Atspi.Action.get_action_description(self.obj, index)

        def getKeyBinding(self, index):
                """
                getKeyBinding: 
                @param : index
                the 0-based index of the action for which a key binding is requested.
                Get the key binding associated with a specific action.
                @return : a string containing the key binding for the specified
                action, or an empty string ("") if none exists.
                """
                return Atspi.Action.get_key_binding(self.obj, index)

        def getName(self, index):
                """
                getName: 
                @param : index
                the index of the action whose name is requested.
                Get the unlocalized name of the specified action. Action names
                generally describe the user action, i.e. "click" or "press",
                rather than the result of invoking the action.
                @return : a string containing the name of the specified action.
                """
                return Atspi.Action.get_action_name(self.obj, index)

        def getLocalizedName(self, index):
                """
                getLocalizedName: 
                @param : index
                the index of the action whose name is requested.
                Get the localized name of the specified action. Action names
                generally describe the user action, i.e. "click" or "press",
                rather than the result of invoking the action.
                @return : a string containing the name of the specified action.
                """
                return Atspi.Action.get_localized_name(self.obj, index)

        def get_nActions(self):
                return Atspi.Action.get_n_actions(self.obj)
        _nActionsDoc = \
                """
                nActions: a long containing the number of actions this object
                supports.
                """
        nActions = property(fget=get_nActions, doc=_nActionsDoc)

#END----------------------------------------------------------------------------



#!/usr/bin/python
#
# caret.py
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.
#
# This example demonstrates how the pyatspi2 text interface can be used.

import pyatspi

def on_caret_move(event):
    if event.source and event.source.getRole() == pyatspi.ROLE_TERMINAL:
        return

    print_text_at_offset(event.source, event.detail1)

def print_text_at_offset(obj, offset):
    try:
        text = obj.queryText()
    except:
        return
    else:
	char, char_start_offset, char_end_offset = text.getTextAtOffset(offset, pyatspi.TEXT_BOUNDARY_CHAR)
	word, word_start_offset, word_end_offset = text.getTextAtOffset(offset, pyatspi.TEXT_BOUNDARY_WORD_START)
	sentence, sentence_start_offset, sentence_end_offset = text.getTextAtOffset(offset, pyatspi.TEXT_BOUNDARY_SENTENCE_START)
	line, line_start_offset, line_end_offset = text.getTextAtOffset(offset,pyatspi.TEXT_BOUNDARY_LINE_START)
	print("\n\nChar:%s \nWord:%s \nSentence:%s Line:%s " % (char, word, sentence, line))

pyatspi.Registry.registerEventListener(on_caret_move, "object:text-caret-moved")
pyatspi.Registry.start()



#Copyright (C) 2010 Novell, Inc.
#Copyright (C) 2008 Codethink Ltd
#copyright: Copyright (c) 2005, 2007 IBM Corporation

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

#Portions of this code originally licensed and copyright (c) 2005, 2007
#IBM Corporation under the BSD license, available at
#U{http://www.opensource.org/licenses/bsd-license.php}

#authors: Peter Parente, Mark Doffman

#------------------------------------------------------------------------------

__all__ = ["Registry",
           "MAIN_LOOP_GLIB",
           "MAIN_LOOP_NONE",
           "set_default_registry"]

import os as _os
from gi.repository import Atspi
from gi.repository import GLib
import time

#------------------------------------------------------------------------------

MAIN_LOOP_GLIB = 'GLib'
MAIN_LOOP_QT   = 'Qt'
MAIN_LOOP_NONE = 'None'

#------------------------------------------------------------------------------

class Registry(object):
        """
        Wraps the Accessibility.Registry to provide more Pythonic registration for
        events.

        This object should be treated as a singleton, but such treatment is not
        enforced. You can construct another instance of this object and give it a
        reference to the Accessibility.Registry singleton. Doing so is harmless and
        has no point.

        @@ivar asynchronous: Should event dispatch to local listeners be decoupled
                from event receiving from the registry?
        @@type asynchronous: boolean
        @@ivar reg: Reference to the real, wrapped registry object
        @@type reg: Accessibility.Registry
        @@ivar dev: Reference to the device controller
        @@type dev: Accessibility.DeviceEventController
        @@ivar clients: Map of event names to client listeners
        @@type clients: dictionary
        @@ivar observers: Map of event names to AT-SPI L{_Observer} objects
        @@type observers: dictionary
        """
        __shared_state = {}

        def __init__(self):
                self.__dict__ = self.__shared_state

                try:
                        if (self.has_implementations):
                                return
                except (AttributeError):
                        pass

                self.has_implementations = False

                self.app_event_register = None
                self.desktop = None

                self.main_loop = GLib.MainLoop()

        def __call__(self):
                """
                @@return: This instance of the registry
                @@rtype: L{Registry}
                """
                return self

        def _set_registry (self, main_loop_type, app_name=None):
                """
                Creates a new 'Registry' object and sets this object
                as the default returned by pyatspi.Registry.

                The default registry (without calling this function) uses the
                GLib main loop with caching. It connects to a registry daemon.

                This function should be called before pyatspi is used if you
                wish to change these defaults.

                @@param main_loop_type: 'GLib', 'None' or 'Qt'. If 'None' is selected then caching
                                       is disabled.

                @@param use_registry: Whether to connect to a registry daemon for device events.
                                     Without this the application to connect to must be declared in the
                                     app_name parameter.

                @@param app_name: D-Bus name of the application to connect to when not using the registry daemon.
                """

                self.has_implementations = True

                self.asynchronous = False	# not fully supported yet
                self.started = False
                self.event_listeners = dict()

        def __getattr__(self, name):
            """
            For backwards compatibility with old API
            """
            if name == 'async':
                return self.asynchronous
            return object.__getattr__(self, name)

        def __setattr__(self, name, value):
            """
            For backwards compatibility with old API
            """
            if name == 'async':
                self.asynchronous = value
            object.__setattr__(self, name, value)

        def _set_default_registry (self):
                self._set_registry (MAIN_LOOP_GLIB)

        def start(self, asynchronous=False, gil=True, **kwargs):
                """
                Enter the main loop to start receiving and dispatching events.

                @@param asynchronous: Should event dispatch be asynchronous
                        (decoupled) from event receiving from the AT-SPI registry?
                @@type asynchronous: boolean
                @@param gil: Add an idle callback which releases the Python GIL for a few
                        milliseconds to allow other threads to run? Necessary if other threads
                        will be used in this process.
                @@type gil: boolean
                """
                if 'async' in kwargs:
                    # support previous API
                    asynchronous = kwargs['async']
                if not self.has_implementations:
                        self._set_default_registry ()
                self.started = True

                if gil:
                        def releaseGIL():
                                try:
                                        time.sleep(1e-2)
                                except KeyboardInterrupt as e:
                                        # store the exception for later
                                        releaseGIL.keyboard_exception = e
                                        self.stop()
                                return True
                        # make room for an exception if one occurs during the 
                        releaseGIL.keyboard_exception = None
                        i = GLib.idle_add(releaseGIL)
                        Atspi.event_main()
                        GLib.source_remove(i)
                        if releaseGIL.keyboard_exception is not None:
                                # raise an keyboard exception we may have gotten earlier
                                raise releaseGIL.keyboard_exception
                else:
                        Atspi.event_main()

                self.started = False

        def stop(self, *args):
                """
                Quits the main loop.
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                Atspi.event_quit()

        def getDesktopCount(self):
                """
                Gets the number of available desktops.

                @@return: Number of desktops
                @@rtype: integer
                """
                return 1

        def getDesktop(self, i):
                """
                Gets a reference to the i-th desktop.

                @@param i: Which desktop to get
                @@type i: integer
                @@return: Desktop reference
                @@rtype: Accessibility.Desktop
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                return Atspi.get_desktop(i)

        def eventWrapper(self, event, callback):
                return callback(event)

        def registerEventListener(self, client, *names):
                """
                Registers a new client callback for the given event names. Supports 
                registration for all subevents if only partial event name is specified.
                Do not include a trailing colon.

                For example, 'object' will register for all object events, 
                'object:property-change' will register for all property change events,
                and 'object:property-change:accessible-parent' will register only for the
                parent property change event.

                Registered clients will not be automatically removed when the client dies.
                To ensure the client is properly garbage collected, call 
                L{deregisterEventListener}.

                @@param client: Callable to be invoked when the event occurs
                @@type client: callable
                @@param names: List of full or partial event names
                @@type names: list of string
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                try:
                        listener = self.event_listeners[client]
                except:
                        listener = self.event_listeners[client] = Atspi.EventListener.new(self.eventWrapper, client)
                for name in names:
                        Atspi.EventListener.register (listener, name)

        def registerEventListenerWithApp(self, client, app, *names):
                """
                Registers a new client callback for the given event names. Supports
                registration for all subevents if only partial event name is specified.
                Do not include a trailing colon.

                For example, 'object' will register for all object events,
                'object:property-change' will register for all property change events,
                and 'object:property-change:accessible-parent' will register only for the
                parent property change event.

                Registered clients will not be automatically removed when the client dies.
                To ensure the client is properly garbage collected, call
                L{deregisterEventListener}.

                @@param client: Callable to be invoked when the event occurs
                @@type client: callable
                @@param names: List of full or partial event names
                @@type names: list of string
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                try:
                        listener = self.event_listeners[client]
                except:
                        listener = self.event_listeners[client] = Atspi.EventListener.new(self.eventWrapper, client)
                for name in names:
                        Atspi.EventListener.register_with_app (listener, name, None, app)

        def deregisterEventListener(self, client, *names):
                """
                Unregisters an existing client callback for the given event names. Supports 
                unregistration for all subevents if only partial event name is specified.
                Do not include a trailing colon.

                This method must be called to ensure a client registered by
                L{registerEventListener} is properly garbage collected.

                @@param client: Client callback to remove
                @@type client: callable
                @@param names: List of full or partial event names
                @@type names: list of string
                @@return: Were event names specified for which the given client was not
                        registered?
                @@rtype: boolean
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                try:
                        listener = self.event_listeners[client]
                except:
                        return
                for name in names:
                        Atspi.EventListener.deregister(listener, name)

        # -------------------------------------------------------------------------------

        # TODO: Remove this hack
        _KEY_PRESSED_EVENT=0
        _KEY_RELEASED_EVENT=1

        def makeSyncType(self, synchronous, preemptive, global_):
                result = Atspi.KeyListenerSyncType.NOSYNC
                if synchronous:
                        result |= Atspi.KeyListenerSyncType.SYNCHRONOUS
                if preemptive:
                        result |= Atspi.KeyListenerSyncType.CANCONSUME
                if global_:
                        result |= Atspi.KeyListenerSyncType.ALL_WINDOWS
                return result

        def makeKind(self, kind):
                result = 0
                for i in kind:
                        result |= (1 << i)
                return result

        def registerKeystrokeListener(self,
                                      client,
                                      key_set=[],
                                      mask=0,
                                      kind=(_KEY_PRESSED_EVENT, _KEY_RELEASED_EVENT),
                                      synchronous=True,
                                      preemptive=True,
                                      global_=False):
                """
                Registers a listener for key stroke events.

                @@param client: Callable to be invoked when the event occurs
                @@type client: callable
                @@param key_set: Set of hardware key codes to stop monitoring. Leave empty
                        to indicate all keys.
                @@type key_set: list of integer
                @@param mask: When the mask is None, the codes in the key_set will be 
                        monitored only when no modifier is held. When the mask is an 
                        integer, keys in the key_set will be monitored only when the modifiers in
                        the mask are held. When the mask is an iterable over more than one 
                        integer, keys in the key_set will be monitored when any of the modifier
                        combinations in the set are held.
                @@type mask: integer, iterable, None
                @@param kind: Kind of events to watch, KEY_PRESSED_EVENT or 
                        KEY_RELEASED_EVENT.
                @@type kind: list
                @@param synchronous: Should the callback notification be synchronous, giving
                        the client the chance to consume the event?
                @@type synchronous: boolean
                @@param preemptive: Should the callback be allowed to preempt / consume the
                        event?
                @@type preemptive: boolean
                @@param global_: Should callback occur even if an application not supporting
                        AT-SPI is in the foreground? (requires xevie)
                @@type global_: boolean
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                try:
                        listener = self.event_listeners[client]
                except:
                        listener = self.event_listeners[client] = Atspi.DeviceListener.new(self.eventWrapper, client)
                syncFlag = self.makeSyncType(synchronous, preemptive, global_)
                if hasattr(mask, '__iter__'):
                        masks = mask
                else:
                        masks = [mask]
                for m in masks:
                        Atspi.register_keystroke_listener(listener,
                                                          key_set,
                                                          m,
                                                          self.makeKind (kind),
                                                          syncFlag)

        def deregisterKeystrokeListener(self,
                                        client,
                                        key_set=[],
                                        mask=0,
                                        kind=(_KEY_PRESSED_EVENT, _KEY_RELEASED_EVENT)):
                """
                Deregisters a listener for key stroke events.

                @@param client: Callable to be invoked when the event occurs
                @@type client: callable
                @@param key_set: Set of hardware key codes to stop monitoring. Leave empty
                        to indicate all keys.
                @@type key_set: list of integer
                @@param mask: When the mask is None, the codes in the key_set will be 
                        monitored only when no modifier is held. When the mask is an 
                        integer, keys in the key_set will be monitored only when the modifiers in
                        the mask are held. When the mask is an iterable over more than one 
                        integer, keys in the key_set will be monitored when any of the modifier
                        combinations in the set are held.
                @@type mask: integer, iterable, None
                @@param kind: Kind of events to stop watching, KEY_PRESSED_EVENT or 
                        KEY_RELEASED_EVENT.
                @@type kind: list
                @@raise KeyError: When the client isn't already registered for events
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                try:
                        listener = self.event_listeners[client]
                except:
                        return

                if hasattr(mask, '__iter__'):
                        masks = mask
                else:
                        masks = [mask]
                for m in masks:
                        Atspi.deregister_keystroke_listener (listener, key_set,
                                m, self.makeKind(kind))

                # TODO: enqueueEvent, etc?

        def generateKeyboardEvent(self, keycode, keysym, kind):
                """
                Generates a keyboard event. One of the keycode or the keysym parameters
                should be specified and the other should be None. The kind parameter is 
                required and should be one of the KEY_PRESS, KEY_RELEASE, KEY_PRESSRELEASE,
                KEY_SYM, KEY_STRING, KEY_LOCKMODIFIERS, or KEY_UNLOCKMODIFIERS.

                @@param keycode: Hardware keycode or None
                @@type keycode: integer
                @@param keysym: Symbolic key string or None
                @@type keysym: string
                @@param kind: Kind of event to synthesize
                @@type kind: integer
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                if keysym is None:
                        keysym = ""
                Atspi.generate_keyboard_event (keycode, keysym, kind)

        def generateMouseEvent(self, x, y, name):
                """
                Generates a mouse event at the given absolute x and y coordinate. The kind
                of event generated is specified by the name. For example, MOUSE_B1P 
                (button 1 press), MOUSE_REL (relative motion), MOUSE_B3D (butten 3 
                double-click).

                @@param x: Horizontal coordinate, usually left-hand oriented
                @@type x: integer
                @@param y: Vertical coordinate, usually left-hand oriented
                @@type y: integer
                @@param name: Name of the event to generate
                @@type name: string
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                Atspi.generate_mouse_event (x, y, name)

        def setReferenceWIndow(accessible):
                """
                Sets the reference window that will be used when
                generateMouseEvent is called. Coordinates will be assumed to
                be relative to this window. This * is needed because, due to
                Wayland's security model, it is not currently possible to
                possible to retrieve global coordinates.
                If NULL is passed, then AT-SPI will use the window that has
                focus at the time that atspi_generate_mouse_event is called.

                @@param accessible: the accessible corresponding to the window
                to select. should be a top-level window with a role of
                pyatspi.ROLE_APPLICATION.
                """
                if not self.has_implementations:
                        self._set_default_registry ()
                Atspi.setReferenceWindow(accessible)

        def pumpQueuedEvents (self):
                """
                Dispatch events that have been queued.
                """
                pass

def set_default_registry (main_loop, app_name=None):
        registry = Registry ()
        registry._set_registry (main_loop, app_name)



#C#-Style Events in Python
#Taken from http://code.activestate.com/recipes/410686/
#Copyright 2005-2008 Zoran Isailovski
#
#This file is distributed under the terms of the PSF (Python Software
#Foundation) license. See the COPYING file in this directory

class Events:
	def __getattr__(self, name):
		if hasattr(self.__class__, '__events__'):
			assert name in self.__class__.__events__, \
					 "Event '%s' is not declared" % name
		self.__dict__[name] = ev = _EventSlot(name)
		return ev
	def __repr__(self): return 'Events' + str(list(self))
	__str__ = __repr__
	def __len__(self): return NotImplemented
	def __iter__(self):
		def gen(dictitems=self.__dict__.items()):
			for attr, val in dictitems:
				if isinstance(val, _EventSlot):
					yield val
		return gen()

#------------------------------------------------------------------------------

class _EventSlot:
	def __init__(self, name):
		self.targets = []
		self.__name__ = name
	def __repr__(self):
		return 'event ' + self.__name__
	def __call__(self, *a, **kw):
		for f in self.targets: f(*a, **kw)
	def __iadd__(self, f):
		self.targets.append(f)
		return self
	def __isub__(self, f):
		while f in self.targets: self.targets.remove(f)
		return self

#------------------------------------------------------------------------------

if __name__ == '__main__':

	class MyEvents(Events):
		__events__ = ('OnChange', )

	class ValueModel(object):
		def __init__(self):
			self.events = MyEvents()
			self.__value = None
		def __set(self, value):
			if (self.__value == value): return
			self.__value = value
			self.events.OnChange()
			##self.events.OnChange2() # would fail
		def __get(self):
			return self.__value
		Value = property(__get, __set, None, 'The actual value')

	class SillyView(object):
		def __init__(self, model):
			self.model = model
			model.events.OnChange += self.DisplayValue
			##model.events.OnChange2 += self.DisplayValue # would raise exeception
		def DisplayValue(self):
			print(self.model.Value)


	model = ValueModel()
	view = SillyView(model)

	print('\n--- Events Demo ---')
	# Events in action
	for i in range(5):
		model.Value = 2*i + 1
	# Events introspection
	print(model.events)
	for event in model.events:
		print(event)



#Copyright (C) 2008 Codethink Ltd

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from pyatspi.text import *
from gi.repository import Atspi
from pyatspi.utils import *

__all__ = [
           "EditableText",
          ]

#------------------------------------------------------------------------------

class EditableText(Text):
        """
        Derived from interface Text, EditableText provides methods for
        modifying textual content of components which support editing.
        EditableText also interacts with the system clipboard via copyText,
        cutText, and pasteText.
        """

        def copyText(self, start, end):
                """
                Copy a range of text into the system clipboard. 
                @param : startPos
                the character offset of the first character in the range of text
                being copied. 
                @param : endPos
                the offset of the first character past the end of the range of
                text being copied.
                """
                return Atspi.EditableText.copy_text(self.obj, start, end)

        def cutText(self, start, end):
                """
                Excise a range of text from a Text object, copying it into the
                system clipboard. 
                @param : startPos
                the character offset of the first character in the range of text
                being cut. 
                @param : endPos
                the offset of the first character past the end of the range of
                text being cut. 
                @return True if the text was successfully cut, False otherwise.
                """
                return Atspi.EditableText.cut_text(self.obj, start, end)

        def deleteText(self, start, end):
                """
                Excise a range of text from a Text object without copying it
                into the system clipboard. 
                @param : startPos
                the character offset of the first character in the range of text
                being deleted. 
                @param : endPos
                the offset of the first character past the end of the range of
                text being deleted. 
                @return True if the text was successfully deleted, False otherwise.
                """
                return Atspi.EditableText.delete_text(self.obj, start, end)

        def insertText(self, position, text, length):
                """
                Insert new text contents into an existing text object at a given
                location, while retaining the old contents. 
                @param : position
                the character offset into the Text implementor's content at which
                the new content will be inserted. 
                @param : text
                a UTF-8 string of which length characters will be inserted into
                the text object's text buffer. 
                @param : length
                the number of characters of text to insert. If the character
                count of text is less than or equal to length, the entire contents
                of text will be inserted.
                @return True if the text content was successfully inserted, False
                otherwise.
                """
                return Atspi.EditableText.insert_text(self.obj, position, text, length)

        def pasteText(self, position):
                """
                Copy the text contents of the system clipboard, if any, into
                a Text object, inserting it at a particular character offset.
                @param : position
                the character offset before which the text will be inserted.
                @return True if the text was successfully pasted into the Text
                object, False otherwise.
                """
                return Atspi.EditableText.paste_text(self.obj, position)

        def setTextContents(self, contents):
                """
                Replace the text contents with a new string, discarding the old
                contents.
                @param : newContents
                a UTF-8 string with which the text object's contents will be
                replaced. 
                @return True if the text content was successfully changed, False
                otherwise.
                """
                return Atspi.EditableText.set_text_contents(self.obj, contents)

#END----------------------------------------------------------------------------



#
# Copyright 2009 Novell, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.
#

import dbus
from gi.repository import GObject
import os.path

from xml.dom import minidom
import os

from pasytest import PasyTest as _PasyTest

import pyatspi
from pyatspi import StateSet

st = [pyatspi.STATE_MULTI_LINE,
      pyatspi.STATE_MODAL,
      pyatspi.STATE_INDETERMINATE,
      pyatspi.STATE_SUPPORTS_AUTOCOMPLETION,
      pyatspi.STATE_VERTICAL,]

class AccessibleTest(_PasyTest):

        __tests__ = ["setup",
                     "test_basic",
                     "test_match_any",
                     "test_role",
                     "teardown",
                     ]

        def __init__(self, bus, path):
                _PasyTest.__init__(self, "Collection", False)
                self._bus = bus
                self._path = path


        def setup(self, test):
                self._registry = pyatspi.Registry()
                print(self._path)
                self._desktop = self._registry.getDesktop(0)
                self._root = pyatspi.findDescendant (self._desktop, lambda x: x.name == "atspi-test-main" and x.getRole() == pyatspi.ROLE_WINDOW)

        def assertObjects(self,test,obj,vars,msg):
                test.assertEqual(len(obj), len(vars) // 2, msg + " length")
                for i in range(0, len(vars), 2):
                        test.assertEqual (vars[i], obj[i//2].name, msg + "name" + "#" + str(i//2))
                        test.assertEqual(vars[i+1], obj[i//2].getRole(), msg + " role" + "#" + str(i//2))

        # Used to help add new tests
        def printAsserts(self,obj,msg):
                print("\t\tself.assertObjects(test,ret,(")
                for i in range(0,len(obj)):
                        print("\t\t\t\"" + obj[i].name + "\", " + str(obj[i].getRole()) + ",")
                print("\t\t), \"", msg, "\")")

        def test_basic(self, test):
                collection = self._root.queryCollection()
                stateSet = pyatspi.StateSet()
                rule = collection.createMatchRule (stateSet.raw(),
                        collection.MATCH_NONE,
                [],     # attributes
                        collection.MATCH_NONE,
                [],     # role
                        collection.MATCH_NONE,
                [],     # interfaces
                        collection.MATCH_NONE,
                        False)

                ret = collection.getMatches (rule, collection.SORT_ORDER_CANONICAL, 5, True)
                self.assertObjects(test,ret,(
                        "gnome-settings-daemon", 79 ,
                        "gnome-panel", 79 ,
                        "Bottom Expanded Edge Panel", 25 ,
                        "Top Expanded Edge Panel", 25 ,
                        "nautilus", 79 ,
                ), " 1 ")

                ret = collection.getMatches (rule, collection.SORT_ORDER_REVERSE_CANONICAL, 5, True)
                self.assertObjects(test,ret,(
                        "nautilus", 79,
                        "Top Expanded Edge Panel", 25,
                        "Bottom Expanded Edge Panel", 25,
                        "gnome-panel", 79,
                        "gnome-settings-daemon", 79,
                ), " reverse canonical ")

                obj=ret[2]
                ret = collection.getMatchesTo (obj, rule, collection.SORT_ORDER_REVERSE_CANONICAL, collection.TREE_INORDER, True, 5, True)
                print("--ret:", len(ret))
                self.assertObjects(test,ret,(
                        "gnome-settings-daemon", 79,
                        "gnome-panel", 79,
                ), " getMatchesTo ")
                ret = collection.getMatchesTo (obj, rule, collection.SORT_ORDER_REVERSE_CANONICAL, collection.TREE_INORDER, True, 1, True)
                self.assertObjects(test,ret,(
                        "gnome-panel", 79,
                ), " getMatchesTo w/count=1")
                ret = collection.getMatchesFrom (obj, rule, collection.SORT_ORDER_REVERSE_CANONICAL, collection.TREE_INORDER, 5, True)
                self.assertObjects(test,ret,(
                        "tracker-applet", 79,
                        "metacity", 79,
                        "Desktop", 25,
                        "nautilus", 79,
                        "Top Expanded Edge Panel", 25,
                ), " getMatchesFrom ")
                obj = self._root.getChildAtIndex(1)
                ret = collection.getMatchesFrom (obj, rule, collection.SORT_ORDER_CANONICAL, collection.TREE_RESTRICT_CHILDREN, 5, True)
                self.assertObjects(test,ret,(
                        "Top Expanded Edge Panel", 25,
                ), " Restrict Children ")

        def test_match_any(self, test):
                collection = self._root.queryCollection()
                stateSet = pyatspi.StateSet()
                rule = collection.createMatchRule (stateSet.raw(),
                        collection.MATCH_ANY,
                [],     # attributes
                        collection.MATCH_ANY,
                [],     # role
                        collection.MATCH_ANY,
                [],     # interfaces
                        collection.MATCH_NONE,
                        False)

                ret = collection.getMatches (rule, collection.SORT_ORDER_CANONICAL, 5, True)
                self.assertObjects(test,ret,(
                        "gnome-settings-daemon", 79 ,
                        "gnome-panel", 79 ,
                        "Bottom Expanded Edge Panel", 25 ,
                        "Top Expanded Edge Panel", 25 ,
                        "nautilus", 79 ,
                ), " 1 ")

        def test_role(self, test):
                collection = self._root.queryCollection()
                stateSet = pyatspi.StateSet()

                rule = collection.createMatchRule (stateSet.raw(),
                        collection.MATCH_NONE,
                [],     # attributes
                        collection.MATCH_NONE,
                [pyatspi.ROLE_RADIO_MENU_ITEM],
                        collection.MATCH_ANY,
                [],     # interfaces
                        collection.MATCH_NONE,
                        False)

                ret = collection.getMatches (rule, collection.SORT_ORDER_CANONICAL, 5, True)
                self.assertObjects(test,ret,(
                        "Activity Indicator", 45,
                        "Back", 45,
                        "Forward", 45,
                        "", 45,
                        "Reload", 45,
                ), " role ")

                rule = collection.createMatchRule (stateSet.raw(),
                        collection.MATCH_NONE,
                [],     # attributes
                        collection.MATCH_NONE,
                [pyatspi.ROLE_ENTRY, pyatspi.ROLE_HTML_CONTAINER],
                        collection.MATCH_ANY,
                [],     # interfaces
                        collection.MATCH_NONE,
                        False)

                ret = collection.getMatches (rule, collection.SORT_ORDER_CANONICAL, 5, True)
                self.assertObjects(test,ret,(
                        "gnome-settings-daemon", 79,
                        "gnome-panel", 79,
                        "Bottom Expanded Edge Panel", 25,
                        "Top Expanded Edge Panel", 25,
                        "nautilus", 79,
                        ), " role #2")

        def teardown(self, test):
                pass



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "Value",
          ]

#------------------------------------------------------------------------------

class Value(interface):
        """
        An interface supporting controls which allow a one-dimensional,
        scalar quantity to be modified or which reflect a scalar quantity.
        (If STATE_EDITABLE is not present, the valuator is treated as
        "read only".
        """

        def get_currentValue(self):
                return Atspi.Value.get_current_value(self.obj)
        def set_currentValue(self, value):
                Atspi.Value.set_current_value(self.obj, value)
        _currentValueDoc = \
                """
                The current value of the valuator.
                """
        currentValue = property(fget=get_currentValue, fset=set_currentValue, doc=_currentValueDoc)

        def get_maximumValue(self):
                return Atspi.Value.get_maximum_value(self.obj)
        _maximumValueDoc = \
                """
                The maximum value allowed by this valuator.
                """
        maximumValue = property(fget=get_maximumValue, doc=_maximumValueDoc)

        def get_minimumIncrement(self):
                return Atspi.Value.get_minimum_increment(self.obj)
        _minimumIncrementDoc = \
                """
                The smallest incremental change which this valuator allows. If
                0, the incremental changes to the valuator are limited only by
                the precision of a double precision value on the platform.
                """
        minimumIncrement = property(fget=get_minimumIncrement, doc=_minimumIncrementDoc)

        def get_minimumValue(self):
                return Atspi.Value.get_minimum_value(self.obj)
        _minimumValueDoc = \
                """
                The minimum value allowed by this valuator.
                """
        minimumValue = property(fget=get_minimumValue, doc=_minimumValueDoc)

#END----------------------------------------------------------------------------



#Copyright (C) 2008 Codethink Ltd

#his program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from .Pasy import PasyTestStep, PasyTestFunc, PasyTest
